#pragma once
#include <new>
#include <iostream>
#include "cstdio"
#include "MyException.h"
using namespace std;


template <class T>
struct ListNode
{
   T value;
   ListNode *next;
   ListNode() { next = nullptr; } 
};

template <class T>
class StackList
{
  public:
    friend struct ListNode<T>;

    StackList() : top(nullptr), size(0) {} // { top = nullptr; size = 0; }
    StackList(const StackList &st);
    ~StackList();

    bool Push(const T &x);
    bool Pop(T & x);
    bool Del();

    T & Top();
    const T & Top() const;

    int Size() const { return size;}  // bool Empty() const;

    template <class P>           // cout << *this;
    friend ostream & operator<<(ostream & os, const StackList<P> & st);

    void Print(FILE *f);

  private:
    ListNode<T> *top;
    int size;
};

template <class T>
StackList<T>::~StackList()
{
   while (top) { Del(); }
}

                                       
template <class T>
bool StackList<T>::Push(const T &x)
{
 try {   
     ListNode<T> *p = new ListNode<T>;
     p->value = x;
     p->next = top;
     top = p;
     ++size;
   } catch (std::bad_alloc) {
     return false; 
   }
   return true;
}

template <class T>
bool StackList<T>::Pop(T &x)
{
    if (size == 0) return false;
    ListNode<T> *p = top;
    x = top->value;
    top = top->next;
    delete p;
    --size;
    return true;
}

template <class T>
bool StackList<T>::Del()
{
    if (size == 0) return false;
    ListNode<T> *p = top;
    top = top->next;
    delete p;
    --size;
    return true;
}

template <class T>           // cout << mystack << anotherstack;
ostream & operator<<(ostream & os, const StackList<T> & st)
{
    os << "stack: ";
    for (ListNode<T> *p = st.top; p; p = p->next)
    {
       os << p->value << " ";
    }
    os << std::endl;
    return os;
}


template <class T>
void StackList<T>::Print(FILE *f)
{
    if (!f) return;
    fprintf(f, "stack: ");
    for (ListNode<T> *p = top; p; p = p->next)
    {
       p->value.Print(f);
    }
    fprintf(f, "\n");
}

template <>
void StackList<int>::Print(FILE *f)
{
    if (!f) return;
    fprintf(f, "print stack: ");
    for (ListNode<int> *p = top; p; p = p->next)
    {
       fprintf(f, "%d ", p->value);
    }
    fprintf(f, "\n");
}













