#pragma once
#include <new>
#include <iostream>
#include "MyException.h"
using namespace std;


template <class T>
class StackList
{
  public:
    StackList() : top(nullptr), size(0) {}
    StackList(const StackList &st);
    ~StackList();

    bool Push(const T &x);
    bool Pop(T & x);
    bool Del();

    T & Top();
    const T & Top() const;

    int Size() { return size;}

    template <class P>
    friend ostream & operator<<(ostream& os, const StackList<P> & st);

  private:

    struct ListNode                
    {
       T value;
       ListNode *next;
       ListNode() { next = 0; } 
    };

    StackList<T>::ListNode *top;
    int size;                      
};

template <class T>
StackList<T>::~StackList()
{
   while (top) { Del(); }
}
                                       
template <class T>
bool StackList<T>::Push(const T &x)
{
 try {   
     StackList<T>::ListNode *p = new StackList<T>::ListNode;
     p->value = x;
     p->next = top;
     top = p;
     ++size;
   } catch (std::bad_alloc) {
     return false; 
   }
   return true;
}

template <class T>
bool StackList<T>::Pop(T &x)
{
    if (size == 0) return false;
    StackList<T>::ListNode *p = top;
    x = top->value;
    top = top->next;
    delete p;
    --size;
    return true;
}

template <class T>
bool StackList<T>::Del()
{
    if (size == 0) return false;
    StackList<T>::ListNode *p = top;
    top = top->next;
    delete p;
    --size;
    return true;
}

template <class T>
ostream & operator<<(ostream & os, const StackList<T> & st)
{
    os << "stack: ";
    //for (StackList<T>::ListNode *p = st.top; p; p = p->next)   // 
    for (typename StackList<T>::ListNode *p = st.top; p; p = p->next)
    {
       os << p->value << " ";
    }
    os << std::endl;
    return os;
}

